package org.rcisoft.core.service.impl;

import org.rcisoft.core.exception.ServiceException;
import org.rcisoft.core.result.ResultExceptionEnum;
import org.rcisoft.core.service.AuthenticationService;
import org.rcisoft.core.util.JwtUtil;
import org.rcisoft.sys.user.dao.SysUserMapper;
import org.rcisoft.sys.user.entity.SysUser;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;

/**
 * Created by lcy on 17/11/24.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
public class AuthenticationServiceImpl implements AuthenticationService {

    @Autowired
    private AuthenticationManager authenticationManager;
    @Autowired
    private UserDetailsService userDetailsService;
    @Autowired
    private SysUserMapper sysUserMapper;
    @Autowired
    private PasswordEncoder passwordEncoder;



    @Value("${jwt.tokenHead}")
    private String tokenHead;

    @Transactional(propagation = Propagation.REQUIRED,readOnly = false)
    @Override
    public void register(SysUser userToAdd) {
        Example example = new Example(SysUser.class);
        Example.Criteria criteria = example.createCriteria();
        List<SysUser> sysUser = sysUserMapper.selectByExample(example);
        if (sysUser.size() > 0)
            throw new ServiceException(ResultExceptionEnum.USER_EXISTS);
        userToAdd.setPassword(passwordEncoder.encode(userToAdd.getPassword()));
        int result = sysUserMapper.insertSelective(userToAdd);
        if(result==0)
            throw new ServiceException(ResultExceptionEnum.SAVE_ERROR);
    }

    @Override
    public String login(String username, String password) {
        UsernamePasswordAuthenticationToken upToken = new UsernamePasswordAuthenticationToken(username, password);
        /*进入到 UserDetailsService(JwtUserDetailServiceImpl)  loadUserByUsername 方法*/
        final Authentication authentication = authenticationManager.authenticate(upToken);
        SecurityContextHolder.getContext().setAuthentication(authentication);
        final UserDetails userDetails = userDetailsService.loadUserByUsername(username);
        final String token = JwtUtil.generateToken(userDetails);
        return token;
    }

    @Override
    public String refresh(String oldToken) {
        return null;
    }
}
